package MLibs1
{
	import flash.display.Bitmap;
	import flash.events.TimerEvent;
	import flash.ui.Keyboard;
	import flash.utils.getTimer;
	import flash.utils.Timer;
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.events.KeyboardEvent;
	import starling.text.TextField;
	import starling.events.Event;
	import starling.textures.Texture;
	import MLibs1.MTime;

	public class MStarlingEngine extends Sprite
	{
		public static var engine:MStarlingEngine;
		
		private var maintimer:Timer;
		private var timeiteratelast:Number;
		private var timestep:Number;
		private var iteratecount:uint;
		
		public var keystates:Array;
		public var keytimes:Array;
		
		public var inputkey:Array;
		
		public function MStarlingEngine() 
		{
			engine = this;
			
			maintimer = new Timer(1, 0);
			maintimer.addEventListener(TimerEvent.TIMER, onMainTimer);
			maintimer.start();
			
			timeiteratelast = MTime.MTimeYS();
			SetTimeStep(1000000 / 60);
			
			iteratecount = 0;
			
			
			addEventListener(Event.ADDED_TO_STAGE, onMAddedToStage);
            addEventListener(Event.REMOVED_FROM_STAGE, onMRemovedFromStage);
			addEventListener(Event.ENTER_FRAME, onMEnterFrame);
            
			
			keystates = new Array();
			keytimes = new Array();
			var keyCode:uint;
			for (keyCode = 0; keyCode < 256; ++keyCode)
			{
				keystates[keyCode] = 0;
				keytimes[keyCode] = 0;
			}
			
			inputkey = new Array();
		}
		
		private function onMAddedToStage(event:Event):void
        {
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onMKeyDown);
            stage.addEventListener(KeyboardEvent.KEY_UP, onMKeyUp);
			
			onAddedToStage(event);
        }
        
        private function onMRemovedFromStage(event:Event):void
        {
			stage.removeEventListener(KeyboardEvent.KEY_DOWN, onMKeyDown);
            stage.removeEventListener(KeyboardEvent.KEY_UP, onMKeyUp);

			onRemovedFromStage(event);
        }
		
		public function onAddedToStage(event:Event):void
		{
			// override this
		}
		
        public function onRemovedFromStage(event:Event):void
        {
			// override this
		}

		public function SetTimeStep(newtimestep:Number):void
		{
			timestep = newtimestep;
		}
		
		private function onMainTimer(event:flash.events.TimerEvent):void
		{
			var now:Number = MTime.MTimeYS();
			// we can't use while here because for the case when it iterates twice, no input gets processed in between
			// and it might be cruicial to process the input fast
			if (now >= timeiteratelast + timestep)
			{
				Iterate();
				++iteratecount;
				timeiteratelast += timestep;
			}

			Process();
		}
		
		private function onMEnterFrame(event:Event):void
		{
			Render();
		}
		
		public function Process():void
		{
			// override this
			// gets executed as often as possible when nothing else has to be done
		}

		public function Iterate():void
		{
			// override this
			// gets executed according to the timestep
		}

		public function Render():void
		{
			// override this
			// gets executed before a new frame gets rendered
		}
		
		
		

		private function onMKeyDown(event:KeyboardEvent):void
		{
			onKeyDown(event);
			if (event.keyCode < 256)
			{
				if (keystates[event.keyCode] == 0)
				{
					keystates[event.keyCode] = -1;
					keytimes[event.keyCode] = iteratecount;
					inputkey.push(event.keyCode);
				}
			}
		}

		private function onMKeyUp(event:KeyboardEvent):void
		{
			onKeyUp(event);
			if (event.keyCode < 256)
			{
				if (keystates[event.keyCode] == -1)
				{
					keystates[event.keyCode] = 0;
					keytimes[event.keyCode] = iteratecount;
				}
			}
		}
		
		public function IsPressed(keyCode:uint):Boolean
		{
			return keystates[keyCode] != 0;
		}
		public function IsReleased(keyCode:uint):Boolean
		{
			return keystates[keyCode] == 0;
		}
		public function HasJustBeenPressed(keyCode:uint, additionalcycles:uint = 0):Boolean
		{
			return (keystates[keyCode] != 0) && (keytimes[keyCode] <= iteratecount + additionalcycles);
		}
		public function HasJustBeenReleased(keyCode:uint, additionalcycles:uint = 0):Boolean
		{
			return (keystates[keyCode] == 0) && (keytimes[keyCode] <= iteratecount + additionalcycles);
		}
		
		
		
		public function onKeyDown(event:KeyboardEvent):void
		{
			// override this
		}

		public function onKeyUp(event:KeyboardEvent):void
		{
			// override this
		}
	}
}
